// Alex O.	02/16/2010
// based on Example 4-16 (pp. 122)
// in A.Arevalo et. all - "Programming the Cell Broadband Engine"

#include <spu_mfcio.h>
#include <string.h>
#include <stdio.h>

// Macro for waiting to completion of DMA group related to input tag:
// 1. Write tag mask
// 2. Read status which is blocked untill all tag&#8217;s DMA are completed
#define wait_tag(t) mfc_write_tag_mask(1<<t); mfc_read_tag_status_all();

// Local store buffer: DMA address and size alignment:
// - MUST be 16B aligned otherwise a bus error is generated
// - may be 128B aligned to get better performance
// In this case we use 16B becuase we don't care about performance
char str[256]  __attribute__ ((aligned(16)));
// argp - effective address pointer to the string in main storage
// envp - size of string in main memory in bytes

int main( uint64_t spuid , uint64_t argp, uint64_t envp ){
    uint32_t tag_id = mfc_tag_reserve();
    // reserve a tag from the tag manager
    if (tag_id==MFC_TAG_INVALID){
        printf("SPU: ERROR can't allocate tag ID\n"); return -1;
    }

    // get data from main storage to local store
    mfc_get((void *)(str), argp, (uint32_t)envp, tag_id, 0, 0);

    // wait for get command to complete. wait only on this tag_id.
    wait_tag(tag_id);
    printf("SPU: %s\n", str);
    strcpy(str, "Fill in the form: Name: Synergistic Processing Element");
    printf("SPU: %s\n", str);

    // put data to main storage from local store
    mfc_put((void *)(str), argp, (uint32_t)envp, tag_id, 0, 0);

    // wait for put command to complete. wait only on this tag_id.
    wait_tag(tag_id);

    // release the tag from the tag manager
    mfc_tag_release(tag_id);
    return (0);
}
